# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'converters.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_converters(object):
    def setupUi(self, converters):
        converters.setObjectName("converters")
        converters.resize(1029, 530)
        self.verticalLayout = QtWidgets.QVBoxLayout(converters)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_4 = QtWidgets.QLabel(converters)
        self.label_4.setObjectName("label_4")
        self.verticalLayout.addWidget(self.label_4)
        self.tw_converters = QtWidgets.QTableWidget(converters)
        self.tw_converters.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tw_converters.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tw_converters.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tw_converters.setObjectName("tw_converters")
        self.tw_converters.setColumnCount(3)
        self.tw_converters.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tw_converters.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tw_converters.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tw_converters.setHorizontalHeaderItem(2, item)
        self.verticalLayout.addWidget(self.tw_converters)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.pb_add_converter = QtWidgets.QPushButton(converters)
        self.pb_add_converter.setObjectName("pb_add_converter")
        self.horizontalLayout.addWidget(self.pb_add_converter)
        self.pb_modify_converter = QtWidgets.QPushButton(converters)
        self.pb_modify_converter.setObjectName("pb_modify_converter")
        self.horizontalLayout.addWidget(self.pb_modify_converter)
        self.pb_delete_converter = QtWidgets.QPushButton(converters)
        self.pb_delete_converter.setObjectName("pb_delete_converter")
        self.horizontalLayout.addWidget(self.pb_delete_converter)
        self.pb_load_from_file = QtWidgets.QPushButton(converters)
        self.pb_load_from_file.setObjectName("pb_load_from_file")
        self.horizontalLayout.addWidget(self.pb_load_from_file)
        self.pb_load_from_repo = QtWidgets.QPushButton(converters)
        self.pb_load_from_repo.setObjectName("pb_load_from_repo")
        self.horizontalLayout.addWidget(self.pb_load_from_repo)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_2 = QtWidgets.QLabel(converters)
        self.label_2.setMinimumSize(QtCore.QSize(120, 0))
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_3.addWidget(self.label_2)
        self.le_converter_name = QtWidgets.QLineEdit(converters)
        self.le_converter_name.setObjectName("le_converter_name")
        self.horizontalLayout_3.addWidget(self.le_converter_name)
        spacerItem1 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_3 = QtWidgets.QLabel(converters)
        self.label_3.setMinimumSize(QtCore.QSize(120, 0))
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_5.addWidget(self.label_3)
        self.le_converter_description = QtWidgets.QLineEdit(converters)
        self.le_converter_description.setObjectName("le_converter_description")
        self.horizontalLayout_5.addWidget(self.le_converter_description)
        spacerItem2 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label = QtWidgets.QLabel(converters)
        self.label.setObjectName("label")
        self.verticalLayout_3.addWidget(self.label)
        self.pb_code_help = QtWidgets.QPushButton(converters)
        self.pb_code_help.setObjectName("pb_code_help")
        self.verticalLayout_3.addWidget(self.pb_code_help)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem3)
        self.horizontalLayout_2.addLayout(self.verticalLayout_3)
        self.pteCode = QtWidgets.QPlainTextEdit(converters)
        font = QtGui.QFont()
        font.setFamily("Monospace")
        self.pteCode.setFont(font)
        self.pteCode.setObjectName("pteCode")
        self.horizontalLayout_2.addWidget(self.pteCode)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.pb_save_converter = QtWidgets.QPushButton(converters)
        self.pb_save_converter.setObjectName("pb_save_converter")
        self.verticalLayout_2.addWidget(self.pb_save_converter)
        self.pb_cancel_converter = QtWidgets.QPushButton(converters)
        self.pb_cancel_converter.setObjectName("pb_cancel_converter")
        self.verticalLayout_2.addWidget(self.pb_cancel_converter)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem4)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem5)
        self.pb_cancel_widget = QtWidgets.QPushButton(converters)
        self.pb_cancel_widget.setObjectName("pb_cancel_widget")
        self.horizontalLayout_4.addWidget(self.pb_cancel_widget)
        self.pbOK = QtWidgets.QPushButton(converters)
        self.pbOK.setObjectName("pbOK")
        self.horizontalLayout_4.addWidget(self.pbOK)
        self.verticalLayout.addLayout(self.horizontalLayout_4)

        self.retranslateUi(converters)
        QtCore.QMetaObject.connectSlotsByName(converters)

    def retranslateUi(self, converters):
        _translate = QtCore.QCoreApplication.translate
        converters.setWindowTitle(_translate("converters", "Time converters"))
        self.label_4.setText(_translate("converters", "Converters"))
        self.tw_converters.setSortingEnabled(True)
        item = self.tw_converters.horizontalHeaderItem(0)
        item.setText(_translate("converters", "Name"))
        item = self.tw_converters.horizontalHeaderItem(1)
        item.setText(_translate("converters", "Description"))
        item = self.tw_converters.horizontalHeaderItem(2)
        item.setText(_translate("converters", "Code"))
        self.pb_add_converter.setText(_translate("converters", "Add new converter"))
        self.pb_modify_converter.setText(_translate("converters", "Modify converter"))
        self.pb_delete_converter.setText(_translate("converters", "Delete converter"))
        self.pb_load_from_file.setText(_translate("converters", "Load converters from file"))
        self.pb_load_from_repo.setText(_translate("converters", "Load converters from BORIS repository"))
        self.label_2.setText(_translate("converters", "Name"))
        self.label_3.setText(_translate("converters", "Description"))
        self.label.setText(_translate("converters", "Python code"))
        self.pb_code_help.setText(_translate("converters", "Help"))
        self.pb_save_converter.setText(_translate("converters", "Save converter"))
        self.pb_cancel_converter.setText(_translate("converters", "Cancel"))
        self.pb_cancel_widget.setText(_translate("converters", "Cancel"))
        self.pbOK.setText(_translate("converters", "OK"))

